
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifndef	M_PI
#define M_PI	3.14159265358979323846
#endif

#define LEN	1024

int	main(int argc,char **argv)
{
	FILE	*file;
        char	str[LEN];
	double	x0,y0,z0,d,
		l,cl,sl,p,cp,sp,cx,cy,cz,c,c1,c2,Cl,Sl,Cp,Sp,Cx,Cy,Cz,C1,C2,D,
		t,ct,st,bx,by,bz,b,b1,b2,Ct,St,Bx,By,Bz,B1,B2,
		Ax,Ay,Az,a,A1,A2,
		*x=NULL,*y=NULL,*z=NULL,step=1.0;
	int	n,N=0;

	if (argc!=2 && argc!=3) {
	    (void)fputs("usage : zcp2abc_lis file {step}\n",stderr); exit(1);
	}
	if (argc==3 && (step=atof(argv[2]))<=0.0) {
	    (void)fputs("bad step of angle in degree.\n",stderr); exit(1);
	}
	step*=(M_PI/180.0);

	if (argv[1][0]=='-')
	    file=stdin;
	else
	    if ((file=fopen(argv[1],"r"))==NULL) {
		(void)fputs("file not found.\n",stderr); exit(1);
	    }

	while (fgets(str,LEN,file)!=NULL) {
	    if (sscanf(str,"%lf %lf %lf %lf",&x0,&y0,&z0,&d)!=3) continue;

	    n=N++;
	    if ((x=(double *)realloc(x,sizeof(double)*N))==NULL ||
		(y=(double *)realloc(y,sizeof(double)*N))==NULL ||
		(z=(double *)realloc(z,sizeof(double)*N))==NULL) {
		(void)fputs("memory allocation error.\n",stderr); exit(1);
	    }
	    x[n]=x0; y[n]=y0; z[n]=z0;
	}
	if (file!=stdin) fclose(file);

	for (l=0.0; l<2.0*M_PI; l+=step) {
	    cl=cos(l); sl=sin(l);
	    for (p=0.0; p<M_PI; p+=step) {
		cp=cos(p-M_PI/2.0); sp=sin(p-M_PI/2.0);
		cx=cl*cp; cy=sl*cp; cz=sp;
		for (n=0; n<N; n++) {
		    c=cx*x[n]+cy*y[n]+cz*z[n];
		    if (n==0) c1=c2=c; else if (c<c1) c1=c; else if (c>c2) c2=c;
		}
		if (l<step/2.0 && p<step/2.0) {
		    Cl=cl; Sl=sl; Cp=cp; Sp=sp;
		    Cx=cx; Cy=cy; Cz=cz; D=(C2=c2)-(C1=c1);
		}
		else if ((d=c2-c1)>D) {
		    Cl=cl; Sl=sl; Cp=cp; Sp=sp;
		    Cx=cx; Cy=cy; Cz=cz; D=d; C1=c1; C2=c2;
		}
	    }
	}
	for (t=0.0; t<M_PI; t+=step) {
	    ct=cos(t); st=sin(t);
	    bx=(-Sl)*st+Cl*Sp*ct; by=Cl*st+Sl*Sp*ct; bz=(-Cp)*ct;
	    for (n=0; n<N; n++) {
		b=bx*x[n]+by*y[n]+bz*z[n];
		if (n==0) b1=b2=b; else if (b<b1) b1=b; else if (b>b2) b2=b;
	    }
	    if (t<step/2.0) {
		Ct=ct; St=st; Bx=bx; By=by; Bz=bz; D=(B2=b2)-(B1=b1);
	    }
	    else if ((d=b2-b1)>D) {
		Ct=ct; St=st; Bx=bx; By=by; Bz=bz; D=d; B1=b1; B2=b2;
	    }
	}
	Ax=Sl*Ct+Cl*Sp*St; Ay=(-Cl)*Ct+Sl*Sp*St; Az=(-Cp)*St;
	for (n=0; n<N; n++) {
	    a=Ax*x[n]+Ay*y[n]+Az*z[n];
	    if (n==0) A1=A2=a; else if (a<A1) A1=a; else if (a>A2) A2=a;
	}
	(void)printf("%lg\t%lg\t%lg\n",A2-A1,B2-B1,C2-C1);

#define X(A,B,C)	A*Ax+B*Bx+C*Cx
#define Y(A,B,C)	A*Ay+B*By+C*Cy
#define Z(A,B,C)	A*Az+B*Bz+C*Cz

	(void)printf("%lg\t%lg\t%lg\n",X(A1,B1,C1),Y(A1,B1,C1),Z(A1,B1,C1));
	(void)printf("%lg\t%lg\t%lg\n",X(A2,B1,C1),Y(A2,B1,C1),Z(A2,B1,C1));
	(void)printf("%lg\t%lg\t%lg\n",X(A2,B2,C1),Y(A2,B2,C1),Z(A2,B2,C1));
	(void)printf("%lg\t%lg\t%lg\n",X(A1,B2,C1),Y(A1,B2,C1),Z(A1,B2,C1));
	(void)printf("%lg\t%lg\t%lg\n",X(A1,B1,C2),Y(A1,B1,C2),Z(A1,B1,C2));
	(void)printf("%lg\t%lg\t%lg\n",X(A2,B1,C2),Y(A2,B1,C2),Z(A2,B1,C2));
	(void)printf("%lg\t%lg\t%lg\n",X(A2,B2,C2),Y(A2,B2,C2),Z(A2,B2,C2));
	(void)printf("%lg\t%lg\t%lg\n",X(A1,B2,C2),Y(A1,B2,C2),Z(A1,B2,C2));

	free(z); free(y); free(x); return 0;
}
